/* -*- c++ -*- */
/////////////////////////////////////////////////////////////////////////////
//
// Particles.h  -- Copyright (c) 2006 David Henry
// last modification: feb. 9, 2006
//
// This code is licenced under the MIT license.
//
// This software is provided "as is" without express or implied
// warranties. You may freely copy and compile this source into
// applications you distribute provided that the copyright text
// below is included in the resulting source code.
//
// Definition of particle systems related classes.
//
/////////////////////////////////////////////////////////////////////////////

#ifndef __PARTICLES_H__
#define __PARTICLES_H__


// Vectors
typedef float vec3_t[3];
typedef float vec4_t[4];

// The Pi constant
#ifdef M_PI
const float kPi = M_PI;
#else
const float kPi = 3.14159265;
#endif

// Gravity constant (in m/s^2)
const float kGravity = 9.81f;

// Frictionnal force at bounce (1.0 = no friction,
// 0.0 = maximum friction)
const float kFriction = 0.5f;



/////////////////////////////////////////////////////////////////////////////
//
// class Particle -- Simple particle class.
//
/////////////////////////////////////////////////////////////////////////////

class Particle
{
public:
  // Constructor/destructor
  Particle ();
  ~Particle ();

public:
  // Public interface
  void init (const vec3_t &pos, const vec3_t &vel, const vec3_t &accel,
	     float life, float size, float weight, const vec4_t &color);

  void update (float dt);
  void drawPoint (bool reflection);
  void drawQuad (const vec3_t &right, const vec3_t &up, bool reflection);
  void drawLight ();

  bool isAlive () const { return _alive; }

private:
  // Member variables
  vec3_t _position;
  vec3_t _velocity;
  vec3_t _acceleration;

  float _life;
  bool _alive;

  float _size;
  float _weight;

  vec4_t _color;
};


/////////////////////////////////////////////////////////////////////////////
//
// class ParticleSystem -- Generic particle system class (abstract).
//
/////////////////////////////////////////////////////////////////////////////

class ParticleSystem
{
public:
  // Constructor/destructor
  ParticleSystem (int maxParticles, float life, const vec3_t &origin);
  virtual ~ParticleSystem ();

public:
  // Public interface
  virtual void	update (float dt) = 0;
  virtual void	render (bool relfection) = 0;

  virtual int emit (int numParticles);
  virtual void startEmitting () { _emitting = true; }
  virtual void stopEmitting () { _emitting = false; }

protected:
  // Internal functions
  virtual void initializeParticle (Particle &p) = 0;

protected:
  // Member variables
  bool _emitting;

  Particle *_particleList;
  vec3_t _origin;

  int _maxParticles;
  int _numParticles;

  float _particleLife;
  float _accumulatedTime;
};


/////////////////////////////////////////////////////////////////////////////
//
// class ParticleFountain -- A particle fountain class.
//
/////////////////////////////////////////////////////////////////////////////

class ParticleFountain : public ParticleSystem
{
public:
  // Constructor
  ParticleFountain (int maxParticles, float life,
	    const vec3_t &origin, const vec3_t &direction);

public:
  // Public interface
  void update (float dt);
  void render (bool reflection);

  int emit (float iNumParticles);

  void setTexture (GLuint texId) { _texId = texId; }

protected:
  // Internal functions
  void initializeParticle (Particle &p);

private:
  // Member variables

  // Emission direction
  vec3_t _emitDir;

  // Number of particles to emit
  float _emit;

  // Texture used for particles
  GLuint _texId;
};

#endif	// __PARTICLES_H__
